<?php
use WHMCS\Database\Capsule;

function Alipay_Connect_config() {
	$configarray = array(
		'name' 			=> 'Alipay Connect',
		'description' 	=> '此模块可以让用户在whmcs上实现alipay快捷登陆,修改自neworld开源插件',
		'version' 		=> '1.1',
		'author' 		=> 'flyqie',
		'fields' 		=> []
	);
	
	$configarray['fields']['alipay_appid'] = [
		'FriendlyName' 	=> 'APP ID',
		'Type' 			=> 'text',
		'Size' 			=> '25'
	];

	$configarray['fields']['alipay_key'] = [
		'FriendlyName' 	=> 'Alipay公钥',
		'Type' 			=> 'textarea',
		'Rows' 			=> '10',
		'Cols' 			=> '60'
	];
	
	$configarray['fields']['alipay_rsa_key'] = [
		'FriendlyName' 	=> 'RSA私钥',
		'Type' 			=> 'textarea',
		'Rows' 			=> '10',
		'Cols' 			=> '60'
	];
	
	return $configarray;
}

function Alipay_Connect_activate() {
	try {
		if (!Capsule::schema()->hasTable('mod_alipayconnect')) {
			Capsule::schema()->create('mod_alipayconnect', function ($table) {
				$table->increments('uid');
				$table->text('openid');
				$table->text('nickname');
				$table->text('avatar');
			});
		}
		if (!Capsule::schema()->hasTable('mod_alipaysetting')) {
			
			Capsule::schema()->create('mod_alipaysetting', function ($table) {
				$table->text('login');
				$table->text('logins');
				$table->text('logout');
			});
			
		}
	} catch (Exception $e) {
		return [
			'status' => 'error',
			'description' => '不能创建表 mod_alipayconnect: ' . $e->getMessage()
		];
	}
	return [
		'status' => 'success',
		'description' => '模块激活成功. 点击 配置 对模块进行设置。'
	];
}

function Alipay_Connect_deactivate() {

	Capsule::schema()->dropIfExists('mod_alipayconnect');
	Capsule::schema()->dropIfExists('mod_alipaysetting');
	
	return [
		'status' => 'success',
		'description' => '模块卸载成功'
	];
}

function Alipay_Connect_output($vars) {
    $systemurl = \WHMCS\Config\Setting::getValue('SystemURL');
    $modulelink = $vars['modulelink'];
    $result = '<link rel="stylesheet" href="'.$systemurl.'/modules/addons/Alipay_Connect/style.css?v3">';
    
    if (isset($_REQUEST['action'])) {
        switch ($_REQUEST['action']) {
            case 'init':
            	$action = Capsule::table('mod_alipaysetting')
		            ->insert([
		            	'login'		=> "&lt;a href=\"javascript:alipay_login('login');\" class=\"btn btn-block btn-alipay\"&gt;&lt;i class=\"fa fa-alipay\"&gt;&lt;/i&gt; alipay&lt;/a&gt;",
		            	'logins' 	=> "&lt;a href=\"javascript:alipay_login('bind');\" class=\"btn btn-sm btn-alipay\"&gt;&lt;i class=\"fa fa-alipay\"&gt;&lt;/i&gt; 绑定alipay&lt;/a&gt;",
		            	'logout'	=> "&lt;a href=\"javascript:if(confirm('您确定取消 alipay 账号绑定吗？'))alipay_login('bind');\" class=\"btn btn-sm btn-alipay\"&gt;&lt;i class=\"fa fa-alipay\"&gt;&lt;/i&gt; 解绑alipay&lt;/a&gt;",
		            ]);
		        if ( $action ) {
                        $alert = alipay_connect_success('<p>初始化按钮样式成功，请在模板文件 clientareahome.tpl 和 login.tpl 中合适的地方加入 </p>
                        	<p>{$alipaylink} 是登录按钮，绑定按钮，解绑按钮，一个按钮多用。</p>
                        	<p>{$avatar} 是头像，{$nickname} 是昵称，例如</p>
                        	<code style="margin-top: 10px;">{if $avatar}
&lt;span class="avatars"&gt;
&lt;img src="{$avatar}" alt="{$nickname}" /&gt;
{/if}
</code>');
                } else {
                    $alert = alipay_connect_error('初始化失败');
                }
            	breeak;
            case 'edit':
            	$setting = Capsule::table('mod_alipaysetting')->first();
                if ( $setting ) {
                    $login 	= $setting->login;
                    $logins = $setting->logins;
                    $logout = $setting->logout;
                }
                $editor = '
<div class="panel-body">
    <form action="'.$modulelink.'" method="post">
      <input type="hidden" name="action" value="submitedit">
      <div class="form-group">
        <label>登录页按钮</label>
        <textarea class="form-control" rows="3" name="login">'.$login.'</textarea>
      </div>
      <div class="form-group">
        <label>客户中心绑定按钮</label>
        <textarea class="form-control" rows="3" name="logins">'.$logins.'</textarea>
      </div>
      <div class="form-group">
        <label>客户中心解除绑定</label>
        <textarea class="form-control" rows="3" name="logout">'.$logout.'</textarea>
      </div>
        <button type="submit" class="btn btn-primary">提交修改</button>
    </form>
</div>';
                break;
            case 'submitedit':
                if (empty($_POST['login']) || empty($_POST['logins']) || empty($_POST['logout'])) {
	                
                    $alert = alipay_connect_error("修改按钮样式失败，不允许修改值为空。");
                    
                } else {
	                
                    $login = html_entity_decode($_POST['login']);
                    $logins = html_entity_decode($_POST['logins']);
                    $logout = html_entity_decode($_POST['logout']);

                    $action = Capsule::table('mod_alipaysetting')
                            ->update([
                            	'login'		=> $login,
                            	'logins' 	=> $logins,
                            	'logout'	=> $logout,
                            ]);
                    if ( $action ) {
                        $alert = alipay_connect_success('<p>修改按钮样式成功，请在模板文件 clientareahome.tpl 和 login.tpl 中合适的地方加入 </p>
                        	<p>{$alipaylink} 是登录按钮，绑定按钮，解绑按钮，一个按钮多用。</p>
                        	<p>{$avatar} 是头像，{$nickname} 是昵称，例如</p>
                        	<code style="margin-top: 10px;">{if $avatar}
&lt;span class="avatars"&gt;
&lt;img src="{$avatar}" alt="{$nickname}" /&gt;
{/if}
</code>');
                    } else {
                        $alert = alipay_connect_error('按钮样式没有修改。');
                    }
                }
                break;
            case 'count':
                $alipayconnect = Capsule::table('mod_alipayconnect')->orderBy('uid','ASC')->get();
                //print_r($alipayconnect);die();
                foreach ($alipayconnect as $key => $value) {
	                $getName = Capsule::table('tblclients')->where('id', $value->uid)->first();
	                $alipayinfo[$key]['name']		= $getName->firstname . ' ' . $getName->lastname;
					$alipayinfo[$key]['id'] 		= $value->uid;
					$alipayinfo[$key]['openid'] 	= $value->openid;
					$alipayinfo[$key]['avatar'] 	= $value->avatar;
					$alipayinfo[$key]['nickname'] 	= $value->nickname;
                	$alipaylist .= "<tr>
					    <td>{$alipayinfo[$key]['id']}</td>
					    <td><a href='clientssummary.php?userid={$alipayinfo[$key]['id']}'>{$alipayinfo[$key]['name']}</a></td>
					    <td>{$alipayinfo[$key]['openid']}</td>
					    <td>{$alipayinfo[$key]['nickname']}</td>
					    <td><img src='{$alipayinfo[$key]['avatar']}' style='width: 36px;' /></td>
					</tr>";
				}
                break;
            default:
                break;
        }
    }
    $count = Capsule::table('mod_alipayconnect')->count();
    if ($count == 0) {
        $count = '暂无记录';
    } else {
        $count = '<a href="'.$modulelink.'&action=count" class="btn btn-info btn-xs">'.$count.'</a>';
    }
    
    $setting = Capsule::table('mod_alipaysetting')->first();
    if ( !$setting ) {
	     $button = '<a href="'.$modulelink.'&action=init" class="btn btn-xs btn-default">初始化按钮</a>';
    } else {
	     $button = '<a href="'.$modulelink.'&action=edit" class="btn btn-xs btn-default">编辑按钮样式</a>';
    }
    
    $header = '<div class="alert alert-info"><strong>回调地址</strong> '.$systemurl.'/modules/addons/Alipay_Connect/oauth/callback.php</div>
    	<a href="'.$modulelink.'" class="btn btn-default btn-xs" style="margin-bottom: 20px;"><i class="fa fa-chevron-circle-left" aria-hidden="true"></i> 返回</a>
    <div class="panel panel-default">';
    if ( $editor ) {
		$result .= '<div class="panel-heading">编辑按钮信息</div>'.$editor;
	} elseif ( $alipaylist ) {
		$result .= '<table class="table">
		    <thead>
			    <tr>
			        <th>UID</th>
			        <th>用户名</th>
			        <th>OPENID</th>
			        <th>alipay昵称</th>
			        <th>alipay头像</th>
			    </tr>
		    </thead>
		    <tbody>
		    '.$alipaylist.'
		    </tbody>
		</table>';
	} else {
		$result .= '<table class="table">
		    <thead>
			    <tr>
			        <th>模块名称</th>
			        <th>绑定数量</th>
			        <th>按钮信息</th>
			    </tr>
		    </thead>
		    <tbody>
				<tr>
				    <td>Alipay_Connect</td>
				    <td>
				        '.$count.'
				    </td>
				    <td>
				        '.$button.'
				    </td>
				</tr>
		    </tbody>
		</table>';
	}
	$footer = '</div>';

    echo $alert.$header.$result.$footer;
}
    function alipay_connect_error($str) {
        return "<div class=\"alert alert-danger alert-dismissible fade in\" role=\"alert\">{$str}</div>";
    }

    function alipay_connect_success($str) {
        return "<div class=\"alert alert-success alert-dismissible fade in\" role=\"alert\">{$str}</div>";
    }